<?php

namespace Drupal\edit_plus_sweet\EventSubscriber\LayoutBuilderBlockForm;

use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Template\Attribute;
use Drupal\Core\Form\FormStateInterface;
use Drupal\layout_builder\LayoutBuilderEvents;
use Drupal\edit_plus_sweet\Events\LayoutBuilderBlockFormAlterEvent;
use Drupal\layout_builder\Event\SectionComponentBuildRenderArrayEvent;

/**
 * Alters the block form to add some elements for styling of a heading/title.
 */
abstract class EnhancedTitleBase extends LayoutBuilderBlockFormBase {

  /**
   * List of title attributes that will be set in config on block form submit.
   *
   * @var array|string[]
   */
  protected static array $configurableProperties = [
    'first_color',
    'second_color',
    'gradient_rotation',
  ];

  public static function getSubscribedEvents() {
    $events = parent::getSubscribedEvents();
    $events[LayoutBuilderEvents::SECTION_COMPONENT_BUILD_RENDER_ARRAY] = [
      'onBuildRender',
      90,
    ];
    return $events;
  }

  /**
   * {@inheritdoc}
   */
  public function formAlter(LayoutBuilderBlockFormAlterEvent $event) {
    $form = $event->getForm();
    $config = $event->getConfiguration();

    $actions = $form['actions'];
    unset($form['actions']);
    static::addTitleOptions($form['settings'], $config);
    $form['actions'] = $actions;

    $form['#submit'] = array_merge([[$this, 'formSubmit']], $form['#submit']);
    $event->setForm($form);
  }

  /**
   * Submit handler that saves title options like size and color into block.
   */
  public function formSubmit(array &$form, FormStateInterface $form_state) {
    static::setTitleOptions($form_state, 'block', $form_state->getValues()['settings']);
  }

  public function onBuildRender(SectionComponentBuildRenderArrayEvent $event) {
    $block = $event->getPlugin();
    if (!$block instanceof BlockPluginInterface) {
      return;
    }

    $valid_block_ids = [$this->blockId, "inline_block:{$this->blockId}"];
    if (!in_array($block->getPluginId(), $valid_block_ids)) {
      return;
    }
    if (empty($block->getConfiguration()['title_size'])) {
      return;
    }

    $config = $block->getConfiguration();
    if (!empty($config['formatter']['type']) && $config['formatter']['type'] !== 'heading') {
      return;
    }

    $build = $event->getBuild();
    $title_attributes = NULL;
    // Field blocks.
    $content = !empty($build['content'][0]) ? $build['content'][0] : NULL;
    if ($build['#base_plugin_id'] == 'field_block' && $content) {
      /** @var \Drupal\Core\Template\Attribute $title_attributes */
      if ($content[0]['#type'] === 'inline_template') {
        $title_attributes = $content[0]['#context']['attributes'];
      }
      elseif (!empty($content[0]['#title']['#context']['attributes'])) {
        $title_attributes = $content[0]['#title']['#context']['attributes'];
      }
      if ($content[0]['#type'] === 'link') {
        $build['content'][0][0]['#attributes']['class'][] = 'no-underline';
      }
    }
    elseif (!empty($build['content']['#block_content'])) {
      $build['content']['#title_attributes'] = new Attribute();
      $title_attributes = $build['content']['#title_attributes'];
    }

    if (!$title_attributes) {
      $title_attributes = new Attribute();
    }
    if ($title_attributes) {
      static::setTitleAttributes($title_attributes, $config);
    }
    $event->setBuild($build);
  }

  /**
   * Adds fields for enhancement of a heading/title to a given form.
   *
   * @param array $form
   * @param array $config
   *
   * @return void
   */
  public static function addTitleOptions(array &$form, array $config) {
    $form['title_size'] = [
      '#type' => 'select',
      '#title' => t('Select title size'),
      '#options' => [
        'text-jumbo1' => t('Jumbo 65px'),
        'text-jumbo2' => t('Jumbo 54px'),
        'text-jumbo3' => t('Jumbo 45px'),
        'text-jumbo4' => t('Jumbo 38px'),
        'text-large1' => t('Large 32px'),
        'text-large2' => t('Large 28px'),
        'text-large3' => t('Large 24px'),
        'text-large4' => t('Large 21px'),
        'text-large5' => t('Large 18px'),
        'text-large6' => t('Large 16px'),
      ],
      '#default_value' => $config['title_size'] ?? 'text-jumbo4',
    ];

    $has_color_adjustment = (!empty($config['formatter']['type']) && $config['formatter']['type'] == 'heading' && !empty($config['first_color']))
      || empty($config['formatter']) && !empty($config['first_color']);
    $form['title_color'] = [
      '#type' => 'checkbox',
      '#title' => t('Set custom title gradient'),
      '#default_value' => $has_color_adjustment,
    ];
    $form['first_color'] = [
      '#type' => 'color',
      '#title' => t('First color'),
      '#default_value' => $config['first_color'] ?? '#000000',
      '#states' => [
        'visible' => [
          '[name="settings[title_color]"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['second_color'] = [
      '#type' => 'color',
      '#title' => t('Second color'),
      '#default_value' => $config['second_color'] ?? '#000000',
      '#states' => [
        'visible' => [
          '[name="settings[title_color]"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['gradient_rotation'] = [
      '#type' => 'number',
      '#title' => t('Rotation'),
      '#description' => t('Enter the rotation of the gradient in degrees between 0 and 360.'),
      '#min' => 0,
      '#max' => 360,
      '#default_value' => $config['gradient_rotation'] ?? '90',
      '#states' => [
        'visible' => [
          '[name="settings[title_color]"]' => ['checked' => TRUE],
        ],
      ],
    ];
  }

  /**
   * Saves provided values into instance of configurable plugin.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param string $property
   * @param array $values
   *
   * @return void
   * @throws \ReflectionException
   */
  public static function setTitleOptions(FormStateInterface $form_state, string $property, array $values) {
    $form_object = $form_state->getFormObject();
    $p = new \ReflectionProperty($form_object, $property);
    $p->setAccessible(TRUE);
    /** @var \Drupal\Component\Plugin\ConfigurableInterface $component */
    $component = $p->getValue($form_object);
    $config = $component->getConfiguration();
    $config['title_size'] = $values['title_size'];
    foreach (static::$configurableProperties as $property) {
      if (in_array($property, self::$configurableProperties)) {
        if (!empty($values['title_color'])) {
          $config[$property] = $values[$property];
        }
        else {
          unset($config[$property]);
        }
      }
      else {
        $config[$property] = $values[$property];
      }
    }

    $component->setConfiguration($config);
  }

  /**
   * Sets attributes based on given config.
   *
   * @param \Drupal\Core\Template\Attribute $title_attributes
   * @param array $config
   *
   * @return void
   */
  public static function setTitleAttributes(Attribute $title_attributes, array $config) {
    $title_size = $config['title_size'] ?? '';
    if ($title_size) {
      $title_attributes->addClass($title_size);
    }
    $first_color = $config['first_color'] ?? '';
    $second_color = $config['second_color'] ?? '';
    $rotation = ($config['gradient_rotation'] ?? '90') . 'deg';
    if ($first_color && $second_color) {
      $title_attributes->setAttribute('style', "background:linear-gradient($rotation, $first_color, $second_color); -webkit-background-clip:text; color:transparent;");
    }
  }

}
